@@ -21,6 +21,9 @@
 #include <libgnome/gnome-i18n.h>
 #include <gtk/gtkmessagedialog.h>
 #include <string.h>
+#include <stdio.h>
+#include <sys/stat.h>
+#include <sys/types.h>
 
 #include "totem.h"
 #include "totem-private.h"
@@ -31,6 +34,34 @@
 
 #define PROPRIETARY_PLUGINS ".gnome2"G_DIR_SEPARATOR_S"totem-addons"
 
+static void
+totem_action_info (char *reason, Totem *totem)
+{
+	GtkWidget *parent, *error_dialog;
+
+	if (totem == NULL)
+		parent = NULL;
+	else
+		parent = totem->win;
+
+	error_dialog =
+		gtk_message_dialog_new (GTK_WINDOW (parent),
+				GTK_DIALOG_MODAL,
+				GTK_MESSAGE_INFO,
+				GTK_BUTTONS_OK,
+				"%s", reason);
+	gtk_dialog_set_default_response (GTK_DIALOG (error_dialog),
+			GTK_RESPONSE_OK);
+	g_signal_connect (G_OBJECT (error_dialog), "destroy", G_CALLBACK
+			(gtk_widget_destroy), error_dialog);
+	g_signal_connect (G_OBJECT (error_dialog), "response", G_CALLBACK
+			(gtk_widget_destroy), error_dialog);
+	gtk_window_set_modal (GTK_WINDOW (error_dialog), TRUE);
+
+	gtk_widget_show (error_dialog);
+}
+
+
 static gboolean
 totem_display_is_local (Totem *totem)
 {
@@ -99,10 +130,45 @@ on_checkbutton1_toggled (GtkToggleButton *togglebutton, Totem *totem)
 		(BACON_VIDEO_WIDGET (totem->bvw), value);
 }
 
-static void              
-on_checkbutton2_toggled (GtkToggleButton *togglebutton, Totem *totem)
+static void
+totem_prefs_set_show_visuals (Totem *totem, gboolean value, gboolean warn)
 {
 	GtkWidget *item;
+
+	gconf_client_set_bool (totem->gc,
+			GCONF_PREFIX"/show_vfx", value, NULL);
+
+	item = glade_xml_get_widget (totem->xml, "tpw_visuals_type_label");
+	gtk_widget_set_sensitive (item, value);
+	item = glade_xml_get_widget (totem->xml,
+			"tpw_visuals_type_optionmenu");
+	gtk_widget_set_sensitive (item, value);
+	item = glade_xml_get_widget (totem->xml, "tpw_visuals_size_label");
+	gtk_widget_set_sensitive (item, value);
+	item = glade_xml_get_widget (totem->xml,
+			"tpw_visuals_size_optionmenu");
+	gtk_widget_set_sensitive (item, value);
+
+	if (warn == FALSE)
+	{
+		bacon_video_widget_set_show_visuals
+			(BACON_VIDEO_WIDGET (totem->bvw), value);
+		return;
+	}
+
+	if (bacon_video_widget_set_show_visuals
+			(BACON_VIDEO_WIDGET (totem->bvw), value) == FALSE)
+	{
+		totem_action_info (_("The change of this setting will only "
+					"take effect for the next movie, or "
+					"when Totem is restarted"),
+				totem);
+	}
+}
+
+static void
+on_checkbutton2_toggled (GtkToggleButton *togglebutton, Totem *totem)
+{
 	gboolean value;
 
 	value = gtk_toggle_button_get_active (togglebutton);
@@ -118,25 +184,7 @@ on_checkbutton2_toggled (GtkToggleButton *togglebutton, Totem *totem)
 		}
 	}
 
-	gconf_client_set_bool (totem->gc, GCONF_PREFIX"/show_vfx", value, NULL);
-
-	item = glade_xml_get_widget (totem->xml, "tpw_visuals_type_label");
-	gtk_widget_set_sensitive (item, value);
-	item = glade_xml_get_widget (totem->xml, "tpw_visuals_type_optionmenu");
-	gtk_widget_set_sensitive (item, value);
-	item = glade_xml_get_widget (totem->xml, "tpw_visuals_size_label");
-	gtk_widget_set_sensitive (item, value);
-	item = glade_xml_get_widget (totem->xml, "tpw_visuals_size_optionmenu");
-	gtk_widget_set_sensitive (item, value);
-
-	if (bacon_video_widget_set_show_visuals
-		(BACON_VIDEO_WIDGET (totem->bvw), value) == FALSE)
-	{
-		totem_action_error (_("The change of this setting will only "
-					"take effect for the next movie, or "
-					"when Totem is restarted"),
-				totem);
-	}
+	totem_prefs_set_show_visuals (totem, value, TRUE);
 }
 
 static void
@@ -155,7 +203,7 @@ on_tvout_toggled (GtkToggleButton *togglebutton, Totem *totem)
 		(BACON_VIDEO_WIDGET (totem->bvw), type);
 
 	if (value == TRUE)
-		totem_action_error (_("Switching on or off this type of TV-Out requires a restart to take effect."), totem);
+		totem_action_info (_("Switching on or off this type of TV-Out requires a restart to take effect."), totem);
 }
 
 static void
@@ -273,21 +321,22 @@ static void
 on_button1_clicked (GtkButton *button, Totem *totem)
 {
 	GError *err = NULL;
-	char *path, *cmd;
+	char *path, *cmd, *filemanager;
 
-	path = path = g_build_path (G_DIR_SEPARATOR_S,
+	path = g_build_path (G_DIR_SEPARATOR_S,
 			g_get_home_dir (), PROPRIETARY_PLUGINS, NULL);
 
-	cmd = g_strdup_printf ("nautilus --no-default-window %s", path);
+	if (g_getenv ("KDE_STARTUP_ENV") && (filemanager = g_find_program_in_path("konqueror"))) {
+		cmd = g_strconcat (filemanager, " ", path, NULL);
+		g_free (filemanager);
+	} else {
+		cmd = g_strdup_printf ("nautilus --no-default-window -no-desktop %s", path);
+	}
 	g_free (path);
 
 	if (g_spawn_command_line_async (cmd, &err) == FALSE)
 	{
-		char *msg;
-
-		msg = g_strdup_printf ("Totem could not start the file manager\nReason: %s.", err->message);
-		totem_action_error (msg, totem);
-		g_free (msg);
+		totem_action_error (_("Totem could not start the file manager."), err->message, totem);
 		g_error_free (err);
 	}
 
@@ -315,7 +364,7 @@ visual_menu_changed (GtkOptionMenu *option_menu, Totem *totem)
 				name, NULL);
 
 		if (bacon_video_widget_set_visuals (totem->bvw, name) == TRUE)
-			totem_action_error (_("Changing the visuals effect type will require a restart to take effect."), totem);
+			totem_action_info (_("Changing the visuals effect type will require a restart to take effect."), totem);
 	}
 }
 
@@ -361,6 +410,14 @@ contrast_changed (GtkRange *range, Totem *totem)
 			BVW_VIDEO_CONTRAST, (int) i);
 }
 
+static void
+audio_out_menu_changed (GtkOptionMenu *option_menu, Totem *totem)
+{
+	BaconVideoWidgetAudioOutType audio_out;
+	audio_out = gtk_option_menu_get_history (GTK_OPTION_MENU (option_menu));
+	bacon_video_widget_set_audio_out_type (totem->bvw, audio_out);
+}
+
 void
 totem_setup_preferences (Totem *totem)
 {
@@ -370,6 +427,7 @@ totem_setup_preferences (Totem *totem)
 	int connection_speed, i, found;
 	char *path, *visual;
 	GList *list, *l;
+	BaconVideoWidgetAudioOutType audio_out;
 
 	g_return_if_fail (totem->gc != NULL);
 
@@ -456,8 +514,7 @@ totem_setup_preferences (Totem *totem)
 
 	gtk_toggle_button_set_active
 		(GTK_TOGGLE_BUTTON (item), show_visuals);
-	bacon_video_widget_set_show_visuals
-		(BACON_VIDEO_WIDGET (totem->bvw), show_visuals);
+	totem_prefs_set_show_visuals (totem, show_visuals, FALSE);
 	g_signal_connect (G_OBJECT (item), "toggled",
 			G_CALLBACK (on_checkbutton2_toggled), totem);
 
@@ -471,7 +528,7 @@ totem_setup_preferences (Totem *totem)
 	if (visual == NULL || strcmp (visual, "") == 0)
 		visual = g_strdup ("goom");
 
-	i = 0;
+	i = found = 0;
 	for (l = list; l != NULL; l = l->next)
 	{
 		const char *name = l->data;
@@ -501,6 +558,7 @@ totem_setup_preferences (Totem *totem)
 	g_signal_connect (G_OBJECT (item), "changed",
 			G_CALLBACK (visual_menu_changed), totem);
 
+	/* Brightness */
 	item = glade_xml_get_widget (totem->xml, "tpw_bright_scale");
 	i = bacon_video_widget_get_video_property (totem->bvw,
 			BVW_VIDEO_BRIGHTNESS);
@@ -508,6 +566,7 @@ totem_setup_preferences (Totem *totem)
 	g_signal_connect (G_OBJECT (item), "value-changed",
 			G_CALLBACK (brightness_changed), totem);
 
+	/* Contrast */
 	item = glade_xml_get_widget (totem->xml, "tpw_contrast_scale");
 	i = bacon_video_widget_get_video_property (totem->bvw,
 			BVW_VIDEO_CONTRAST);
@@ -515,6 +574,13 @@ totem_setup_preferences (Totem *totem)
 	g_signal_connect (G_OBJECT (item), "value-changed",
 			G_CALLBACK (contrast_changed), totem);
 
+	/* Sound output type */
+	item = glade_xml_get_widget (totem->xml, "tpw_sound_output_optionmenu");
+	audio_out = bacon_video_widget_get_audio_out_type (totem->bvw);
+	gtk_option_menu_set_history (GTK_OPTION_MENU (item), audio_out);
+	g_signal_connect (G_OBJECT (item), "changed",
+			G_CALLBACK (audio_out_menu_changed), totem);
+
 	/* This one is for the deinterlacing menu, not really our dialog
 	 * but we do it anyway */
 	item = glade_xml_get_widget (totem->xml, "tmw_deinterlace_menu_item");
@@ -528,6 +594,12 @@ totem_setup_preferences (Totem *totem)
 			totem, NULL, NULL);
 	g_signal_connect (G_OBJECT (item), "activate",
 			G_CALLBACK (on_deinterlace1_activate), totem);
+
+	/* Always On Top */
+	item = glade_xml_get_widget (totem->xml, "tmw_always_on_top_menu_item");
+	gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (item),
+			gconf_client_get_bool (totem->gc,
+				GCONF_PREFIX"/window_on_top", NULL));
 }
 
 void
@@ -551,6 +623,7 @@ totem_preferences_tvout_setup (Totem *totem)
 		break;
 	default:
 		g_assert_not_reached ();
+		name = NULL;
 	}
 
 	item = glade_xml_get_widget (totem->xml, name);
@@ -585,3 +658,4 @@ totem_preferences_visuals_setup (Totem *totem)
 
 	bacon_video_widget_set_visuals (totem->bvw, visual);
 }
+
